import 'package:flutter/material.dart';

import 'context.dart';

final _keys = Context<Object, Key>();

typedef KeyProviderBuilder = Widget Function(
  BuildContext context,
  Key key,
  Widget? child,
);

class KeyProvider<T> extends Widget {
  final Object keyId;
  final KeyProviderBuilder builder;
  final Widget? child;

  String get scopedKey => '$T($keyId)';

  KeyProvider({
    Key? key,
    required this.keyId,
    required this.builder,
    this.child,
  }) : super(key: key);

  @override
  Element createElement() {
    return KeyProviderElement<T>(this);
  }
}

class KeyProviderElement<T> extends ComponentElement {
  KeyProviderElement(KeyProvider widget) : super(widget);

  KeyProvider get widget => super.widget as KeyProvider;

  @override
  void mount(Element? parent, Object? newSlot) {
    _keys[widget.scopedKey] = GlobalKey();
    super.mount(parent, newSlot);
  }

  @override
  void update(KeyProvider newWidget) {
    if (widget.scopedKey != newWidget.scopedKey) {
      _keys.remove(widget.scopedKey);
      _keys[newWidget.scopedKey] = GlobalKey();
      markNeedsBuild();
    }

    super.update(newWidget);
  }

  @override
  void unmount() {
    _keys.remove(widget.scopedKey);
    super.unmount();
  }

  @override
  Widget build() {
    final key = _keys[widget.scopedKey]!;
    return widget.builder(this, key, widget.child);
  }
}
